@@ -15,9 +15,10 @@ from rest_framework.test import APIClient
 
 from . import ADDITIONAL_BASE_UNITS
 from .exceptions import UnitSystemNotFound, UnitDuplicateError, \
-    UnitDimensionError, UnitValueError
+    UnitDimensionError, UnitValueError, DimensionDuplicateError, \
+    DimensionValueError, DimensionDimensionError, DimensionNotFound
 from .models import UnitSystem, UnitConverter, \
-    Dimension, DimensionNotFound, CustomUnit
+    Dimension, DimensionNotFound, CustomUnit, CustomDimension
 from .serializers import QuantitySerializer
 
 
@@ -82,7 +83,7 @@ class DimensionTest(TestCase):
         """
         us = UnitSystem()
         dimension = Dimension(unit_system=us, code='[length]')
-        unit_codes = [unit.code for unit in dimension.units()]
+        unit_codes = [unit.code for unit in dimension.units]
         self.assertIn('meter', unit_codes)
 
     def test_compounded_dimension_units(self):
@@ -91,7 +92,7 @@ class DimensionTest(TestCase):
         """
         us = UnitSystem()
         dimension = Dimension(unit_system=us, code='[compounded]')
-        unit_codes = [unit.code for unit in dimension.units()]
+        unit_codes = [unit.code for unit in dimension.units]
         self.assertIn('number_english', unit_codes)
 
     def test_custom_dimension_superuser_units(self):
@@ -100,7 +101,7 @@ class DimensionTest(TestCase):
         """
         us = UnitSystem(user=self.admin)
         dimension = Dimension(unit_system=us, code='[custom]')
-        self.assertEqual(len(dimension.units(user=self.admin)), 2)
+        self.assertEqual(len(dimension.units), 2)
 
     def test_custom_dimension_superuser_key_units(self):
         """
@@ -109,8 +110,7 @@ class DimensionTest(TestCase):
         """
         us = UnitSystem(user=self.admin, key=str(self.key))
         dimension = Dimension(unit_system=us, code='[custom]')
-        self.assertEqual(len(dimension.units(user=self.admin,
-                                             key=self.key)), 1)
+        self.assertEqual(len(dimension.units), 1)
 
     def test_custom_dimension_user_units(self):
         """
@@ -118,7 +118,7 @@ class DimensionTest(TestCase):
         """
         us = UnitSystem(user=self.user)
         dimension = Dimension(unit_system=us, code='[custom]')
-        self.assertEqual(len(dimension.units(user=self.user)), 2)
+        self.assertEqual(len(dimension.units), 2)
 
     def test_custom_dimension_user_key_units(self):
         """
@@ -126,16 +126,33 @@ class DimensionTest(TestCase):
         """
         us = UnitSystem(user=self.user, key=str(self.key))
         dimension = Dimension(unit_system=us, code='[custom]')
-        self.assertEqual(len(dimension.units(user=self.user,
-                                             key=self.key)), 1)
+        self.assertEqual(len(dimension.units), 1)
 
     def test_custom_dimension_no_user_units(self):
         """
         Test units list if no user
         """
-        us = UnitSystem(user=self.user)
+        us = UnitSystem()
         dimension = Dimension(unit_system=us, code='[custom]')
-        self.assertEqual(len(dimension.units()), 0)
+        self.assertEqual(len(dimension.units), 0)
+
+    def test_dimension_name(self):
+        """
+        Test name transformation
+        """
+        cd = CustomDimension.objects.create(
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code="[myDim]",
+            name="My dimension",
+            relation="[length]/[luminance]"
+        )
+        us = UnitSystem(user=self.user, key=str(self.key))
+        dims = us.available_dimensions()
+        self.assertEqual(dims['[myDim]'].name, 'My dimension')
+        self.assertEqual(dims['[length]'].name, _('length'))
+        self.assertEqual(dims['[esu_magnetic_dipole]'].name, 'Esu magnetic dipole')
 
 
 class UnitTest(TestCase):
@@ -178,7 +195,6 @@ class UnitAPITest(TestCase):
     """
     Test Unit APIs
     """
-
     def test_list_request(self):
         """
         Test list of units
@@ -278,7 +294,35 @@ class UnitAPITest(TestCase):
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         self.assertEqual(len(response.json()),
                          len(Dimension(unit_system=us,
-                                       code='[length]').units()))
+                                       code='[length]').units))
+
+    def test_list_with_domain_request(self):
+        """
+        Test list filtered by dimension
+        """
+        client = APIClient()
+        response = client.get(
+            '/units/SI/units/',
+            data={
+                'domain': 'mydomain'
+            }
+        )
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertEqual(len(response.json()), 1)
+
+    def test_list_with_wrong_domain_request(self):
+        """
+        Test list filtered by dimension
+        """
+        client = APIClient()
+        response = client.get(
+            '/units/SI/units/',
+            data={
+                'domain': 'mywrongdomain'
+            }
+        )
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        self.assertEqual(len(response.json()), 0)
 
     def test_list_with_compounded_dimension_request(self):
         """
@@ -295,7 +339,7 @@ class UnitAPITest(TestCase):
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         self.assertEqual(len(response.json()),
                          len(Dimension(unit_system=us,
-                                       code='[compounded]').units()))
+                                       code='[compounded]').units))
 
     def test_list_with_dimension_2_request(self):
         """
@@ -312,7 +356,7 @@ class UnitAPITest(TestCase):
         self.assertEqual(response.status_code, status.HTTP_200_OK)
         self.assertEqual(len(response.json()),
                          len(Dimension(unit_system=us,
-                                       code='[area]').units()))
+                                       code='[area]').units))
 
     def test_retrieve_request(self):
         """
@@ -522,7 +566,7 @@ class UnitSystemAPITest(TestCase):
         response = client.get(
             '/units/SI/dimensions/',
             data={
-                'ordering': 'dimension'
+                'ordering': '-code'
             }
         )
         self.assertEqual(response.status_code, status.HTTP_200_OK)
@@ -781,6 +825,459 @@ class UnitConverterAPITest(TestCase):
         self.assertEqual(response.json().get('id'), str(batch_id))
 
 
+class CustomDimensionTest(TestCase):
+    """
+    CustomDimension tests
+    """
+
+    def setUp(self) -> None:
+        """
+        Setup environment
+        """
+        self.user, created = User.objects.get_or_create(
+            username='test',
+            email='test@ipd.com'
+        )
+        self.user.set_password('test')
+        self.user.save()
+        Token.objects.create(user=self.user)
+        self.key = uuid.uuid4()
+
+    def test_creation(self):
+        """
+        Test creation of a CustomDimension
+        """
+        cd = CustomDimension.objects.create(
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[my_dimension]',
+            name='My Dimension',
+            relation="[time] * [length]")
+        self.assertEqual(cd.user, self.user)
+        self.assertEqual(cd.key, self.key)
+        self.assertEqual(cd.unit_system, 'SI')
+        self.assertEqual(cd.code, '[my_dimension]')
+        self.assertEqual(cd.name, 'My Dimension')
+        self.assertEqual(cd.relation, '[time] * [length]')
+        self.assertEqual(
+            CustomDimension.objects.filter(
+                user=self.user, key=self.key,
+                unit_system='SI', code='[my_dimension]').count(),
+            1)
+        us = UnitSystem(system_name='SI', user=self.user, key=self.key)
+        self.assertIn('[my_dimension]', us.available_dimension_names())
+        self.assertIn('[my_dimension]', us.available_dimensions().keys())
+
+    def test_unit_assignment(self):
+        """
+        Test units associated with the custom dimension
+        """
+        cd = CustomDimension.objects.create(
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[my_dimension]',
+            name='My Dimension',
+            relation="[time] * [length]")
+        cu = CustomUnit.objects.create(
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='my_unit',
+            name='My Unit',
+            relation="1.5 hour * meter",
+            symbol="myu",
+            alias="myu")
+        us = UnitSystem(system_name='SI', user=self.user, key=self.key)
+        dim = Dimension(unit_system=us, code='[my_dimension]')
+        self.assertIn('my_unit', [u.code for u in dim.units])
+
+    def test_creation_with_dash(self):
+        """
+        Test creation of a CustomDimension with - in code
+        """
+        cd = CustomDimension.objects.create(
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[my-dimension]',
+            name='My Dimension',
+            relation="[time] * [length]"
+        )
+        self.assertEqual(cd.user, self.user)
+        self.assertEqual(cd.key, self.key)
+        self.assertEqual(cd.unit_system, 'SI')
+        self.assertEqual(cd.code, '[my_dimension]')
+        self.assertEqual(cd.name, 'My Dimension')
+        self.assertEqual(cd.relation, '[time] * [length]')
+        self.assertEqual(
+            CustomDimension.objects.filter(
+                user=self.user, key=self.key,
+                unit_system='SI', code='[my_dimension]').count(),
+            1)
+
+    def test_creation_with_wrong_dimension(self):
+        """
+        Test creation of a CustomUnit with - in code, symbol and alias
+        """
+        self.assertRaises(
+            DimensionDimensionError,
+            CustomDimension.objects.create,
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[my_dimension]',
+            name='My Dimension',
+            relation="[time] * [space]"
+        )
+
+    def test_invalid_creation_params(self):
+        """
+        Test invalid creation params
+        """
+        self.assertRaises(
+            UnitSystemNotFound,
+            CustomDimension.objects.create,
+            user=self.user,
+            key=self.key,
+            unit_system='SO',
+            code='[my_dimension]',
+            name='My Dimension',
+            relation="[time] * [length]"
+        )
+
+    def test_duplicate_dimension_params(self):
+        """
+        Test duplicated unit
+        """
+        self.assertRaises(
+            DimensionDuplicateError,
+            CustomDimension.objects.create,
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[length]',
+            name='My Dimension',
+            relation="[length]"
+        )
+
+    def test_duplicate_dimension_relation_params(self):
+        """
+        Test duplicated unit
+        """
+        self.assertRaises(
+            DimensionDuplicateError,
+            CustomDimension.objects.create,
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[mojo]',
+            name='My Dimension',
+            relation="[length]"
+        )
+
+
+class CustomDimensionAPITest(TestCase):
+    """
+    CustomUnit API tests
+    """
+
+    def setUp(self) -> None:
+        """
+        Setup environment
+        """
+        self.user, created = User.objects.get_or_create(
+            username='test',
+            email='test@ipd.com'
+        )
+        self.user.set_password('test')
+        self.user.save()
+        Token.objects.create(user=self.user)
+        self.key = uuid.uuid4()
+
+    def test_list_request(self):
+        """
+        Test list of custom dimensions
+        """
+        client = APIClient()
+        response = client.get(
+            '/units/SI/dimensions/custom/',
+            format='json')
+        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+
+    def test_connected_list_request(self):
+        """
+        Test list of custom dimensions with connected user
+        """
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        post_response = client.post(
+            '/units/SI/dimensions/custom/',
+            data={
+                'code': '[my_dimension]',
+                'name': 'My Dimension',
+                'relation': "[energy] / [time]"
+            }
+        )
+        self.assertEqual(post_response.status_code, status.HTTP_201_CREATED)
+        self.assertIn('code', post_response.json())
+        response = client.get(
+            '/units/SI/dimensions/custom/',
+            format='json')
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        if 'results' in response.json():
+            # Paginated results
+            self.assertEqual(len(response.json()['results']), 1)
+            self.assertEqual(response.json()['results'][0]['code'],
+                             '[my_dimension]')
+        else:
+            # Non paginated results
+            self.assertEqual(len(response.json()), 1)
+            self.assertEqual(response.json()[0]['code'], '[my_dimension]')
+
+    def test_connected_duplicate_post(self):
+        """
+        Test list of custom dimensions with connected user
+        """
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        post_response = client.post(
+            '/units/SI/dimensions/custom/',
+            data={
+                'code': '[my_dimension]',
+                'name': 'My Dimension',
+                'relation': "[energy] / [time]"
+            }
+        )
+        self.assertEqual(post_response.status_code, status.HTTP_201_CREATED)
+        self.assertIn('code', post_response.json())
+        post_response = client.post(
+            '/units/SI/dimensions/custom/',
+            data={
+                'code': '[my_dimension]',
+                'name': 'My Dimension',
+                'relation': "[energy] / [time]"
+            }
+        )
+        self.assertEqual(post_response.status_code, status.HTTP_409_CONFLICT)
+
+    def test_connected_unit_system_request(self):
+        """
+        Test list of custom dimensions with connected user
+        """
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        post_response = client.post(
+            '/units/SI/dimensions/custom/',
+            data={
+                'code': '[my_dimension]',
+                'name': 'My Dimension',
+                'relation': "[energy] / [time]"
+            }
+        )
+        self.assertEqual(post_response.status_code, status.HTTP_201_CREATED)
+        self.assertIn('code', post_response.json())
+        response = client.get(
+            '/units/SI/dimensions/custom/',
+            format='json')
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        if 'results' in response.json():
+            # Paginated results
+            self.assertEqual(len(response.json()['results']), 1)
+            self.assertEqual(response.json()['results'][0]['code'],
+                             '[my_dimension]')
+        else:
+            # Non paginated results
+            self.assertEqual(len(response.json()), 1)
+            self.assertEqual(response.json()[0]['code'], '[my_dimension]')
+        response = client.get(
+            '/units/mks/dimensions/custom/',
+            format='json')
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        if 'results' in response.json():
+            # Paginated results
+            self.assertEqual(len(response.json()['results']), 0)
+        else:
+            # Non paginated results
+            self.assertEqual(len(response.json()), 0)
+
+    def test_connected_list_key_request(self):
+        """
+        Test list of custom dimensions with connected user and a key
+        """
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        post_response = client.post(
+            '/units/SI/dimensions/custom/',
+            data={
+                'key': self.key,
+                'code': '[my_dimension]',
+                'name': 'My Dimension',
+                'relation': "[energy] / [time]"
+            }
+        )
+        self.assertEqual(post_response.status_code, status.HTTP_201_CREATED)
+        self.assertIn('code', post_response.json())
+        response = client.get(
+            '/units/SI/dimensions/custom/',
+            data={'key': self.key},
+            format='json')
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        if 'results' in response.json():
+            # Paginated results
+            self.assertEqual(len(response.json()['results']), 1)
+            self.assertEqual(response.json()['results'][0]['code'],
+                             '[my_dimension]')
+        else:
+            # Non paginated results
+            self.assertEqual(len(response.json()), 1)
+            self.assertEqual(response.json()[0]['code'], '[my_dimension]')
+
+    def test_connected_dimension_list_request(self):
+        """
+        Test list of dimensions with custom dimension and connected user
+        """
+        CustomDimension.objects.create(
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[ny_dimension]',
+            name='Ny Dimension',
+            relation="[energy] / [mass] * [time]")
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        response = client.get(
+            '/units/SI/dimensions/',
+        )
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        if 'results' in response.json():
+            self.assertIn('[ny_dimension]', [u['code']
+                                      for u in response.json()['results']])
+        else:
+            self.assertIn('[ny_dimension]',
+                          [u['code'] for u in response.json()])
+
+    def test_connected_dimension_list_2_request(self):
+        """
+        Another test of a list of dimensions with a custom dimension
+        """
+        new_key = uuid.uuid4()
+        CustomDimension.objects.create(
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[ny_dimension]',
+            name='Ny Dimension',
+            relation="[energy] * [conductance] / [time]")
+        CustomDimension.objects.create(
+            user=self.user,
+            key=new_key,
+            unit_system='SI',
+            code='[py_dimension]',
+            name='Py Dimension',
+            relation="[energy] * [conductance] / [length]")
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        response = client.get(
+            '/units/SI/dimensions/',
+        )
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        if 'results' in response.json():
+            self.assertIn('[ny_dimension]', [u['code']
+                                      for u in response.json()['results']])
+            self.assertIn('[py_dimension]', [u['code']
+                                      for u in response.json()['results']])
+        else:
+            self.assertIn('[ny_dimension]', [u['code'] for u in response.json()])
+            self.assertIn('[py_dimension]', [u['code'] for u in response.json()])
+
+    def test_connected_dimension_list_new_key_request(self):
+        """
+        Test key isolation
+        """
+        new_key = uuid.uuid4()
+        CustomDimension.objects.create(
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[ny_dimension]',
+            name='Ny Unit',
+            relation="[time] * [temperature] * [length]")
+        CustomDimension.objects.create(
+            user=self.user,
+            key=new_key,
+            unit_system='SI',
+            code='[py_dimension]',
+            name='Py Dimension',
+            relation="[volume] / [area]")
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        response = client.get(
+            '/units/SI/dimensions/custom/',
+            data={
+                'key': new_key
+            }
+        )
+
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        if 'results' in response.json():
+            self.assertNotIn('[ny_dimension]', [u['code']
+                                             for u in response.json()['results']])
+            self.assertIn('[py_dimension]', [u['code']
+                                             for u in response.json()['results']])
+        else:
+            self.assertNotIn('[ny_dimension]', [u['code'] for u in response.json()])
+            self.assertIn('[py_dimension]', [u['code'] for u in response.json()])
+
+
+
+    def test_connected_dimension_list_self_key_request(self):
+        """
+        Another isolation test
+        """
+        new_key = uuid.uuid4()
+        CustomDimension.objects.create(
+            user=self.user,
+            key=self.key,
+            unit_system='SI',
+            code='[ny_dimension]',
+            name='Ny Dimension',
+            relation="[time] * [length]")
+        CustomDimension.objects.create(
+            user=self.user,
+            key=new_key,
+            unit_system='SI',
+            code='[py_dimension]',
+            name='Py Dimension',
+            relation="[conductance] * [resistance]")
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        response = client.get(
+            '/units/SI/dimensions/custom/',
+            data={
+                'key': self.key
+            }
+        )
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        if 'results' in response.json():
+            self.assertIn('[ny_dimension]', [u['code']
+                                                for u in response.json()['results']])
+            self.assertNotIn('[py_dimension]', [u['code']
+                                             for u in response.json()['results']])
+        else:
+            self.assertIn('[ny_dimension]', [u['code'] for u in response.json()])
+            self.assertNotIn('[py_dimension]', [u['code'] for u in response.json()])
+
+
 class CustomUnitTest(TestCase):
     """
     CustomUnit tests
@@ -922,7 +1419,7 @@ class CustomUnitTest(TestCase):
         )
 
 
-class CustomUnitTestAPI(TestCase):
+class CustomUnitAPITest(TestCase):
     """
     CustomUnit API tests
     """
@@ -1084,6 +1581,92 @@ class CustomUnitTestAPI(TestCase):
             self.assertEqual(len(response.json()), 1)
             self.assertEqual(response.json()[0]['code'], 'my_unit')
 
+    def test_connected_unit_creation_with_dimension(self):
+        """
+        Test creation of a unit with correct dimension
+        """
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        post_response = client.post(
+            '/units/SI/custom/',
+            data={
+                'key': self.key,
+                'code': 'my_unit',
+                'name': 'My Unit',
+                'relation': "1.5 meter",
+                'symbol': "myu",
+                'alias': "myu",
+                'dimension': '[length]'
+            }
+        )
+        self.assertEqual(post_response.status_code, status.HTTP_201_CREATED)
+        self.assertIn('code', post_response.json())
+        response = client.get(
+            '/units/SI/custom/',
+            format='json')
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        if 'results' in response.json():
+            # Paginated results
+            self.assertEqual(len(response.json()['results']), 1)
+            self.assertEqual(response.json()['results'][0]['code'], 'my_unit')
+        else:
+            # Non paginated results
+            self.assertEqual(len(response.json()), 1)
+            self.assertEqual(response.json()[0]['code'], 'my_unit')
+
+    def test_connected_unit_creation_with_wrong_dimension(self):
+        """
+        Test creation of a unnit with an incompatible dimension
+        """
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        post_response = client.post(
+            '/units/SI/custom/',
+            data={
+                'key': self.key,
+                'code': 'my_unit',
+                'name': 'My Unit',
+                'relation': "1.5 meter",
+                'symbol': "myu",
+                'alias': "myu",
+                'dimension': '[time]'
+            }
+        )
+        self.assertEqual(post_response.status_code, status.HTTP_400_BAD_REQUEST)
+
+    def test_connected_unit_creation_with_new_dimension(self):
+        """
+        Test creation of a unit and association with a new dimension
+        """
+        client = APIClient()
+        token = Token.objects.get(user__username=self.user.username)
+        client.credentials(HTTP_AUTHORIZATION='Token ' + token.key)
+        post_response = client.post(
+            '/units/SI/custom/',
+            data={
+                'key': self.key,
+                'code': 'my_unit',
+                'name': 'My Unit',
+                'relation': "1.5 meter * hour / kelvin",
+                'symbol': "myu",
+                'alias': "myu",
+                'dimension': '[new_dim]'
+            }
+        )
+        self.assertEqual(post_response.status_code, status.HTTP_201_CREATED)
+        us = UnitSystem(system_name='SI', user=self.user, key=self.key)
+        response = client.get(
+            '/units/SI/units/',
+            data={
+                'key': self.key,
+                'dimension': '[new_dim]'
+            }
+        )
+        self.assertEqual(response.json()[0]['code'], 'my_unit')
+
+
     def test_connected_unit_list_request(self):
         """
         Test list of units with custom unit and connected user
@@ -1182,8 +1765,14 @@ class CustomUnitTestAPI(TestCase):
             }
         )
         self.assertEqual(response.status_code, status.HTTP_200_OK)
-        self.assertNotIn('ny_unit', [u['code'] for u in response.json()])
-        self.assertIn('py_unit', [u['code'] for u in response.json()])
+        if 'results' in response.json():
+            self.assertNotIn('ny_unit', [u['code']
+                                         for u in response.json()['result']])
+            self.assertIn('py_unit', [u['code']
+                                      for u in response.json()['result']])
+        else:
+            self.assertNotIn('ny_unit', [u['code'] for u in response.json()])
+            self.assertIn('py_unit', [u['code'] for u in response.json()])
 
     def test_connected_unit_list_self_key_request(self):
         """
@@ -1218,5 +1807,11 @@ class CustomUnitTestAPI(TestCase):
             }
         )
         self.assertEqual(response.status_code, status.HTTP_200_OK)
-        self.assertIn('ny_unit', [u['code'] for u in response.json()])
-        self.assertNotIn('py_unit', [u['code'] for u in response.json()])
+        if 'results' in response.json():
+            self.assertIn('ny_unit', [u['code']
+                                      for u in response.json()['result']])
+            self.assertNotIn('py_unit', [u['code']
+                                         for u in response.json()['result']])
+        else:
+            self.assertIn('ny_unit', [u['code'] for u in response.json()])
+            self.assertNotIn('py_unit', [u['code'] for u in response.json()])
